function [Exp,Exp01,Exp10,Exp20,Var,Var10,Var01,Var20,Var1010,Var0101,Var1001,Var1020,Var0120,Var2020]=Cholesky(f,r,n1,n2,T1,T2)

% Compute Cholesky factors and expectations.
%Varijkl denotes the Covariance between the (i+j)-order partial derivative of X(s,t) in s (i times) and t (j times) and the (k+l)-order partial derivative of X(s,t) in s (k times) and t (l times).
% For instance, Var1001 denotes the Covariance between the first-order partial derivative of X(s,t) in s and the first-order partial derivative of X(s,t) in t.
%Expij denotes the expectation of the (i+j)-order partial derivative of X(s,t) in s (i times) and t (j times).

h = 1e-3; 
dl1 = T1*[0:n1]/n1;
dl2 = T2*[0:n2]/n2;


Exp = zeros(n1+1,n2+1); Exp01 = Exp; Exp10 = Exp; Exp20 = Exp;
Var1010 = zeros(n1+1,n2+1); Var0101 = Var1010; Var1001 = Var1010; Var1020 = Var1010;
Var2020 = Var1010; Var0120 = Var1010;
dl3 = zeros(1,(n1+1)*(n2+1));dl4 = dl3;
Var = zeros((n1+1)*(n2+1)); Var10 = Var; Var01 = Var; Var20 = Var;

for i = 1 : n1+1,
            Var1010(i,1:n2+1)= (r(dl1(i) + h,dl2,dl1(i) + h,dl2) + r(dl1(i),dl2,dl1(i),dl2) - 2*r(dl1(i) + h,dl2,dl1(i),dl2))/h^2;
            Var0101(i,1:n2+1)= (r(dl1(i),dl2 + h,dl1(i),dl2 + h) + r(dl1(i),dl2,dl1(i),dl2) - 2*r(dl1(i),dl2 + h,dl1(i),dl2))/h^2;
            Var1001(i,1:n2+1)= (r(dl1(i) + h,dl2,dl1(i),dl2 + h) + r(dl1(i),dl2,dl1(i),dl2) - r(dl1(i) + h,dl2,dl1(i),dl2) - r(dl1(i),dl2,dl1(i),dl2 + h))/h^2;
            Var1020(i,1:n2+1)= (3*r(dl1(i) + h,dl2,dl1(i),dl2) + r(dl1(i) + h,dl2,dl1(i) + 2*h,dl2) - 2*r(dl1(i)+h,dl2 ,dl1(i) + h,dl2) - r(dl1(i) ,dl2,dl1(i) + 2*h,dl2) - r(dl1(i),dl2,dl1(i),dl2))/h^3;
            Var2020(i,1:n2+1)= (r(dl1(i),dl2,dl1(i) + 2*h,dl2) + r(dl1(i) + 2*h,dl2,dl1(i) + 2*h,dl2) + 4*r(dl1(i) + h,dl2,dl1(i) + h,dl2) + r(dl1(i),dl2,dl1(i),dl2) + r(dl1(i) + 2*h,dl2,dl1(i),dl2) - 2*r(dl1(i) + h,dl2,dl1(i) + 2*h,dl2) - 2*r(dl1(i),dl2,dl1(i) + h,dl2)- 2*r(dl1(i) + 2*h,dl2,dl1(i) + h,dl2) - 2*r(dl1(i) + h,dl2,dl1(i),dl2))/h^4;
            Var0120(i,1:n2+1)= (r(dl1(i),dl2 + h,dl1(i),dl2) + r(dl1(i),dl2 + h,dl1(i) + 2*h,dl2) + 2*r(dl1(i),dl2,dl1(i) + h,dl2) - r(dl1(i),dl2,dl1(i) + 2*h,dl2) - r(dl1(i),dl2,dl1(i),dl2) - 2*r(dl1(i),dl2 + h,dl1(i) + h,dl2))/h^3;
            Exp(i,1:n2+1) = f(dl1(i),dl2);
            Exp01(i,1:n2+1) = (f(dl1(i),dl2 + h)-f(dl1(i),dl2))/h;
            Exp10(i,1:n2+1) = (f(dl1(i) + h,dl2)-f(dl1(i),dl2))/h;
            Exp20(i,1:n2+1) = (f(dl1(i) + 2*h,dl2)-2*f(dl1(i) + h,dl2)+ f(dl1(i),dl2))/h^2;
            dl4(((i-1)*(n2+1) + 1):i*(n2+1)) = dl2;
            dl3(((i-1)*(n2+1) + 1):i*(n2+1)) = dl1(i)*ones(1,n2+1);
            
    for j = 1:n2+1,
                Var((i-1)*(n2+1) + j,1:((i-1)*(n2+1) + j)) = r(dl1(i),dl2(j),dl3(1:((i-1)*(n2+1) + j)),dl4(1:((i-1)*(n2+1) + j)));
                Var01((i-1)*(n2+1) + j,1:((i-1)*(n2+1) + j)) = (r(dl1(i),dl2(j) + h,dl3(1:((i-1)*(n2+1) + j)),dl4(1:((i-1)*(n2+1) + j))) - r(dl1(i),dl2(j),dl3(1:((i-1)*(n2+1) + j)),dl4(1:((i-1)*(n2+1) + j))))/h;
                Var10((i-1)*(n2+1) + j,1:((i-1)*(n2+1) + j)) = (r(dl1(i) + h,dl2(j),dl3(1:((i-1)*(n2+1) + j)),dl4(1:((i-1)*(n2+1) + j))) - r(dl1(i),dl2(j),dl3(1:((i-1)*(n2+1) + j)),dl4(1:((i-1)*(n2+1) + j))))/h;
                Var20((i-1)*(n2+1) + j,1:((i-1)*(n2+1) + j)) = (r(dl1(i) + 2*h,dl2(j),dl3(1:((i-1)*(n2+1) + j)),dl4(1:((i-1)*(n2+1) + j))) + r(dl1(i),dl2(j),dl3(1:((i-1)*(n2+1) + j)),dl4(1:((i-1)*(n2+1) + j))) - 2*r(dl1(i)+h,dl2(j),dl3(1:((i-1)*(n2+1) + j)),dl4(1:((i-1)*(n2+1) + j))))/h^2;
    end
end
end