function [pv11,ev11,p_S01]=p_11(S,m,u,n,Var,Var10,Var1010,Exp,Exp10)


%   This function uses a modification of an algorithm in the paper
%      "Numerical Computation of Multivariate Normal Probabilities", 
%      A. Genz, J. of Comp. Graph. Stat., 1(1992), pp. 141-149.
%
%   The primary references for the numerical integration are 
%    "Randomization of Number Theoretic Methods for Multiple Integration",
%     R. Cranley & T.N.L. Patterson, SIAM J Numer Anal, 13(1976), pp. 904-14.
%   and  
%    "Fast Component-by-Component Construction, a Reprise for Different 
%     Kernels", D. Nuyens & R. Cools. In H. Niederreiter and D. Talay,
%     editors, Monte-Carlo and Quasi-Monte Carlo Methods 2004, 
%     Springer-Verlag, 2006, 371-385.

eps1 = 5e-9;

pv11 = zeros(1,n);
ev11 = zeros(1,n);
df = zeros(S,n);

A = [abs(Var(1,1)),Var10(1,1),abs(Var1010(1,1))];
ch0(1,1)= sqrt(A(1)); ch0(2,1) = A(2)/ch0(1,1);ch0(2,2) = sqrt(abs(A(3) - ch0(2,1)^2));
y_1 = (u - Exp(1,1))/ch0(1,1);
p_S01 = phi(y_1)/ch0(1,1)*((Exp10(1,1) + ch0(2,1)*y_1)*(1-Phi2((-Exp10(1,1) - ch0(2,1)*y_1)/ch0(2,2))) + ch0(2,2)*phi((-Exp10(1,1) - ch0(2,1)*y_1)/ch0(2,2)));


for i=1:n,
        Sg = zeros(i+2);
        Sg(1,1)= Var(i+1,i+1); 
        Sg(2,1)=Var10(i+1,i+1); Sg(1,2) = Sg(2,1); Sg(2,2)=1;%Var1010(i+1,1);
        Sg(3:i+2,1) = Var(i+1,1:i)'; Sg(1,3:i+2) = Sg(3:i+2,1)';
        Sg(3:i+2,2)= Var10(i+1,1:i)'; Sg(2,3:i+2) = Sg(3:i+2,2)';
        Sg(3:i+2,3:i+2) = Var(1:i,1:i) + Var(1:i,1:i)' - diag(diag(Var(1:i,1:i))); %Covariance matrix
        [ U, D ] = svd(Sg);
        d = sqrt(diag(D));
        nc =i+2;
        if d(nc) < eps1, 
            nc = min( find( d < eps1 ) );
        end
        [ Q, R ] = qr( diag(d(1:nc))*U(:,1:nc)' );
        ch = ( diag(sign(diag(R)))*R )';  %Cholesky for R_i
       
        
        [q pr] = fstrnk(m,nc-1);  %Lattice Rule 
         q = q/pr;
        
        uv = (u - Exp(i+1,1))/ch(1,1)*ones(1,m);
        N_1 = Phi2((-Exp10(i+1,1)-ch(2,1)*(u-Exp(i+1,1))/ch(1,1))/ch(2,2));
        pu = phi(uv)*(1-N_1);
        
        for K = 1:S,
            fi = zeros(1,m); y = zeros(nc-2,m);
            y(1,:) = Phinv(abs(2*mod(q(1)*[1:m]+rand,1) - 1 )*(1-N_1) + N_1);
            fi = pu.*(ch(2,2)*y(1,:)+ ch(2,1)*uv + Exp10(i+1,1)*ones(1,m))/ch(1,1);
            
                    bi = 9;
                    ai = -9;
                for j = 2 : nc-2, 
                    ys = ch(j+1,1:j)*[uv;y(1:j-1,:)];
                    dc = Phi2( min( ( u - Exp(j-1,1) - ys )/ch(j+1,j+1), 9 ) );
                    fi = fi.*dc;
                    y(j,:) = Phinv(abs( 2*mod(q(j)*[1:m]+rand,1) - 1 ).*dc ); % MCQMC
                end
               
                 for j = nc:i+2, 
                     
                    ys = ch(j,1:nc-1)*[ uv; y(1:nc-2,:) ];
                      if ch(j,nc) > 0,
                           bi = min( ( u - Exp(j-1,1) - ys )/ch(j,nc), bi );
                           else
                           ai = max( ( u - Exp(j-1,1) - ys )/ch(j,nc), ai );
                      end
                 end
                
            df(K,i) = mean(fi.*(Phi2(bi) - Phi2(ai))); %Estimation of R_i 
        end
%         Sg = zeros(min(i+1,3));
%         Sg(1,1)= Var(i+1,i+1); 
%         Sg(2,1)=Var10(i+1,i+1); Sg(1,2) = Sg(2,1); Sg(2,2)=1;%Var1010(i+1,1);
%         if not(i==1),
%         Sg(3,1) = Var(i+1,1); Sg(1,3) = Sg(3,1);
%         Sg(3,2)= Var10(i+1,1:1); Sg(2,3) = Sg(3,2);
%         Sg(3,3) = Var(1:1,1:1); %Covariance matrix
%         end
%         [ U, D ] = svd(Sg);
%         d = sqrt(diag(D));
%         nc =min(i+1,3);
%         if d(nc) < eps1, 
%             nc = min( find( d < eps1 ) );
%         end
%         [ Q, R ] = qr( diag(d(1:nc))*U(:,1:nc)' );
%         ch = ( diag(sign(diag(R)))*R )';  %Cholesky for R_i
%        
%         
%         [q pr] = fstrnk(m,nc-1);  %Lattice Rule 
%          q = q/pr;
%         
%         uv = u/ch(1,1)*ones(1,m);
%         N_1 = Phi2((-Exp10(i+1,1)-ch(2,1)*(u-Exp(i+1,1))/ch(1,1))/ch(2,2));
%         pu = phi(uv)*(1-N_1);
%         
%         for K = 1:S,
%             fi = zeros(1,m); y = zeros(nc-2,m);
%             y(1,:) = Phinv(abs(2*mod(q(1)*[1:m]+rand,1) - 1 )*(1-N_1) + N_1);
%             fi = pu.*(ch(2,2)*y(1,:)+ ch(2,1)*uv + Exp10(i+1,1)*ones(1,m))/ch(1,1);
%             
%                     bi = 9;
%                     ai = -9;
%                 for j = 2 : nc-2, 
%                     ys = ch(j+1,1:j)*[uv;y(1:j-1,:)];
%                     dc = Phi2( min( ( u - Exp(j-1,1) - ys )/ch(j+1,j+1), 9 ) );
%                     fi = fi.*dc;
%                     y(j,:) = Phinv(abs( 2*mod(q(j)*[1:m]+rand,1) - 1 ).*dc ); % MCQMC
%                 end
%                
%                  for j = nc:min(i+1,3), 
%                      
%                     ys = ch(j,1:nc-1)*[ uv; y(1:nc-2,:) ];
%                       if ch(j,nc) > 0,
%                            bi = min( ( u - Exp(j-1,1) - ys )/ch(j,nc), bi );
%                            else
%                            ai = max( ( u - Exp(j-1,1) - ys )/ch(j,nc), ai );
%                       end
%                  end
%                 
%             df(K,i) = mean(fi.*(Phi2(bi) - Phi2(ai))); %Estimation of R_i 
%         end
end
pv11 = mean(df);
ev11 = 1/(S*(S-1))*sum((df - ones(S,1)*pv11).^2); 
ev11 = 3*sqrt(ev11);
end
