function [pv2,ev2]=p_22(S,m,u,n1,n2,m1,m2,Var,Var10,Var01,Var20,Var1010,Var1001,Var1020,Var0101,Var0120,Var2020,Exp,Exp01,Exp10,Exp20)


%   This function uses a modification of an algorithm in the paper
%      "Numerical Computation of Multivariate Normal Probabilities", 
%      A. Genz, J. of Comp. Graph. Stat., 1(1992), pp. 141-149.
%
%   The primary references for the numerical integration are 
%    "Randomization of Number Theoretic Methods for Multiple Integration",
%     R. Cranley & T.N.L. Patterson, SIAM J Numer Anal, 13(1976), pp. 904-14.
%   and  
%    "Fast Component-by-Component Construction, a Reprise for Different 
%     Kernels", D. Nuyens & R. Cools. In H. Niederreiter and D. Talay,
%     editors, Monte-Carlo and Quasi-Monte Carlo Methods 2004, 
%     Springer-Verlag, 2006, 371-385.

eps1 = 5e-9; 
pv2 = zeros((n1+1)*(n2+1));
ev2 = zeros((n1+1)*(n2+1));
df = zeros(S,(n1+1)*(n2+1));

q1 = floor(n1/m1);
q2 = floor(n2/m2);
        
for i=0:n1,
    for j=0:n2,
        Sg = zeros((floor((i-1)/q1)+1)*(floor((j-1)/q2)+1)+4); 
        Sg(1,1)= Var((n2+1)*i+j+1,(n2+1)*i+j+1); Sg(2,1)= Var10((n2+1)*i+j+1,(n2+1)*i+j+1); 
        Sg(3,1)= Var01((n2+1)*i+j+1,(n2+1)*i+j+1); Sg(4,1)= Var20((n2+1)*i+j+1,(n2+1)*i+j+1);
        Sg(2,2)= Var1010(i+1,j+1); Sg(3,2)= Var1001(i+1,j+1);
        Sg(4,2)= Var1020(i+1,j+1);
        Sg(3,3)= Var0101(i+1,j+1); Sg(4,3)= Var0120(i+1,j+1); Sg(4,4)=1;% Var2020(i+1,j+1);
        
        if not((i==0)||(j==0)),
                b=5;
                for k=1:1+floor(i/q1), 
                    v = b + floor(j/q2) + (k<=floor(i/q1));
                    Sg(1,b:v)= Var(i*(n2+1)+ j + 1,(k-1)*q1*(n2+1)+1:q2:((k-1)*q1*(n2+1)+ 1 + q2*(floor(j/q2) + (k<=floor(i/q1)))))';
                    Sg(2,b:v)= Var10(i*(n2+1)+ j + 1,(k-1)*q1*(n2+1)+1:q2:((k-1)*q1*(n2+1)+ 1 + q2*(floor(j/q2) + (k<=floor(i/q1)))))';
                    Sg(3,b:v)= Var01(i*(n2+1)+ j + 1,(k-1)*q1*(n2+1)+1:q2:((k-1)*q1*(n2+1)+ 1 + q2*(floor(j/q2) + (k<=floor(i/q1)))))';
                    Sg(4,b:v)= Var20(i*(n2+1)+ j + 1,(k-1)*q1*(n2+1)+1:q2:((k-1)*q1*(n2+1)+ 1 + q2*(floor(j/q2) + (k<=floor(i/q1)))))';
                    c=b;
                    for l=k:floor(i/q1)+1,
                        w=c+floor(j/q2)+(l<=floor(i/q1));
                        Sg(c:w,b:v) = Var((l-1)*q1*(n2+1)+1:q2:((l-1)*q1*(n2+1)+1+ q2*(floor(j/q2) + (l<=floor(i/q1)))),(k-1)*q1*(n2+1)+1:q2:((k-1)*q1*(n2+1)+1+ q2*(floor(j/q2) + (k<=floor(i/q1)))));
                        c=w+1;
                    end
                    b=v+1;

                end
                
        end
        Sg = Sg + Sg' - diag(diag(Sg));
        %Covariance matrix
        
        [ U, D ] = svd(Sg);
        d = sqrt(diag(D));
        nc =length(Sg);
        if d(nc) < eps1, 
            nc = min( find( d < eps1 ) );
        end
        [ Q, R ] = qr( diag(d(1:nc))*U(:,1:nc)' );
        ch = ( diag(sign(diag(R)))*R )';  %Cholesky for K_{i,j}
        [q m] = fstrnk(m,nc-1);  %Lattice Rule 
         q = q/m;
        y_1 = (u - Exp(i+1,j+1))/ch(1,1);
        y_2 = (-Exp10(i+1,j+1) - ch(2,1)*y_1)/ch(2,2);
        uv1 = y_1*ones(1,m);
        uv2 = y_2*ones(1,m);
        
        N_1 = Phi2((-Exp01(i+1,j+1)-ch(3,1)*y_1 - ch(3,2)*y_2)/ch(3,3));
        pu = phi(y_1)*phi(uv2)*(1-N_1);
       
        for K = 1:S,
            fi = zeros(1,m); y = zeros(nc-2,m);
            y(1,:) = Phinv(abs(2*mod(q(1)*[1:m]+rand,1) - 1 )*(1-N_1) + N_1);
            N_2 = Phi2((-Exp20(i+1,j+1) - ch(4,1)*uv1 - ch(4,2)*uv2 - ch(4,3)*y(1,:))/ch(4,4));
            y(2,:) = Phinv(abs(2*mod(q(2)*[1:m]+rand,1) - 1 ).*N_2);
            fi = -pu.*N_2.*(Exp01(i+1,j+1) + ch(3,1)*uv1 + ch(3,2)*uv2 + ch(3,3)*y(1,:)).*(Exp20(i+1,j+1) + ch(4,1)*uv1 + ch(4,2)*uv2 + ch(4,3)*y(1,:) + ch(4,4)*y(2,:))/ch(1,1);

                      bi = 9;
                      ai = -9;
               if not((i==0)&(j==0)),
                for k = 5 : nc-1,
                    ys = ch(k,1:k-1)*[uv1;uv2;y(1:k-3,:)];
                    dc = Phi2( min( ( u - ys )/ch(k,k), 9) );
                    fi = fi.*dc;
                    y(k-2,:) = Phinv(abs( 2*mod(q(k-2)*[1:m]+rand,1) - 1 ).*dc ); % MCQMC
                end

                 for k = nc:length(Sg),
                    ys = ch(k,1:nc-1)*[ uv1; uv2; y(1:nc-3,:) ];
                      if ch(k,nc) > 0,
                           bi = min( ( u -  ys )/ch(k,nc), bi );
                      else
                           ai = max( ( u - ys )/ch(k,nc), ai );
                      end
                 end
               end
            df(K,i*(n2+1) + j + 1) = mean(fi.*(Phi2(max(ai,bi)) - Phi2(min(ai,bi)))); %Estimation of K_{i,j} 
        end
    end

end
pv2 = mean(df);
ev2 = 1/(S*(S-1))*sum((df - ones(S,1)*pv2).^2); ev2 = 3*sqrt(ev2);
end